@@ -24,6 +24,7 @@ import wx
 import itertools
 
 from invesalius import utils
+from invesalius import inv_paths
 
 #from invesalius.project import Project
 INVESALIUS_VERSION = "3.1.1"
@@ -98,9 +99,15 @@ SAGITAL = 3
 VOLUME = 4
 SURFACE = 5
 
+AXIAL_STR="AXIAL"
+CORONAL_STR="CORONAL"
+SAGITAL_STR="SAGITAL"
+
 # Measure type
 LINEAR = 6
 ANGULAR = 7
+DENSITY_ELLIPSE = 8
+DENSITY_POLYGON = 9
 
 # Colour representing each orientation
 ORIENTATION_COLOUR = {'AXIAL': (1,0,0), # Red
@@ -329,59 +336,13 @@ REDUCE_IMAGEDATA_QUALITY = 0
 # PATHS
 FS_ENCODE = sys.getfilesystemencoding()
 
-if sys.platform == 'win32':
-    from invesalius.expanduser import expand_user
-    try:
-        USER_DIR = expand_user()
-    except:
-        USER_DIR = utils.decode(os.path.expanduser('~'), FS_ENCODE)
-else:
-    USER_DIR = utils.decode(os.path.expanduser('~'), FS_ENCODE)
-
-USER_INV_DIR = os.path.join(USER_DIR, u'.invesalius')
-USER_PRESET_DIR = os.path.join(USER_INV_DIR, u'presets')
-USER_LOG_DIR = os.path.join(USER_INV_DIR, u'logs')
-
-FILE_PATH = utils.decode(os.path.split(__file__)[0], FS_ENCODE)
-
-if hasattr(sys,"frozen") and (sys.frozen == "windows_exe"\
-                            or sys.frozen == "console_exe"):
-    abs_path = os.path.abspath(FILE_PATH, u'..', u'..', u'..')
-    ICON_DIR = os.path.join(abs_path, u"icons")
-    SAMPLE_DIR = os.path.join(FILE_PATH, u'samples')
-    DOC_DIR = os.path.join(FILE_PATH, u'docs')
-    folder=RAYCASTING_PRESETS_DIRECTORY= os.path.join(abs_path, u"presets", u"raycasting")
-    RAYCASTING_PRESETS_COLOR_DIRECTORY = os.path.join(abs_path, u"presets", u"raycasting", u"color_list")
-
-else:
-    ICON_DIR = os.path.abspath(os.path.join(FILE_PATH, u'..', u'icons'))
-    SAMPLE_DIR = os.path.abspath(os.path.join(FILE_PATH, u'..', u'samples'))
-    DOC_DIR = os.path.abspath(os.path.join(FILE_PATH, u'..', u'docs'))
-
-    folder=RAYCASTING_PRESETS_DIRECTORY= os.path.abspath(os.path.join(u".",
-                                                                  u"presets",
-                                                                  u"raycasting"))
-
-    RAYCASTING_PRESETS_COLOR_DIRECTORY = os.path.abspath(os.path.join(u".",
-                                                                  u"presets",
-                                                                  u"raycasting",
-                                                                  u"color_list"))
-
-
-# MAC App
-if not os.path.exists(ICON_DIR):
-    ICON_DIR = os.path.abspath(os.path.join(FILE_PATH, u'..', u'..', u'..', u'..', u'icons'))
-    SAMPLE_DIR = os.path.abspath(os.path.join(FILE_PATH, u'..',  u'..', u'..', u'..', u'samples'))
-    DOC_DIR = os.path.abspath(os.path.join(FILE_PATH, u'..', u'..', u'..', u'..', u'docs'))
-
-
-ID_TO_BMP = {VOL_FRONT: [_("Front"), os.path.join(ICON_DIR, u"view_front.png")],
-             VOL_BACK: [_("Back"), os.path.join(ICON_DIR, u"view_back.png")],
-             VOL_TOP: [_("Top"), os.path.join(ICON_DIR, u"view_top.png")],
-             VOL_BOTTOM: [_("Bottom"), os.path.join(ICON_DIR, u"view_bottom.png")],
-             VOL_RIGHT: [_("Right"), os.path.join(ICON_DIR, u"view_right.png")],
-             VOL_LEFT: [_("Left"), os.path.join(ICON_DIR, u"view_left.png")],
-             VOL_ISO:[_("Isometric"), os.path.join(ICON_DIR, u"view_isometric.png")]
+ID_TO_BMP = {VOL_FRONT: [_("Front"), str(inv_paths.ICON_DIR.joinpath("view_front.png"))],
+             VOL_BACK: [_("Back"), str(inv_paths.ICON_DIR.joinpath("view_back.png"))],
+             VOL_TOP: [_("Top"), str(inv_paths.ICON_DIR.joinpath("view_top.png"))],
+             VOL_BOTTOM: [_("Bottom"), str(inv_paths.ICON_DIR.joinpath("view_bottom.png"))],
+             VOL_RIGHT: [_("Right"), str(inv_paths.ICON_DIR.joinpath("view_right.png"))],
+             VOL_LEFT: [_("Left"), str(inv_paths.ICON_DIR.joinpath("view_left.png"))],
+             VOL_ISO:[_("Isometric"), str(inv_paths.ICON_DIR.joinpath("view_isometric.png"))]
              }
 
 # if 1, use vtkVolumeRaycastMapper, if 0, use vtkFixedPointVolumeRayCastMapper
@@ -427,10 +388,9 @@ RAYCASTING_FILES = {_("Airways"): "Airways.plist",
 #                    os.path.isfile(os.path.join(folder,filename))]
 
 
-USER_RAYCASTING_PRESETS_DIRECTORY = os.path.join(USER_PRESET_DIR, u'raycasting')
-RAYCASTING_TYPES = [_(filename.split(".")[0]) for filename in
-                     os.listdir(USER_RAYCASTING_PRESETS_DIRECTORY) if
-                     os.path.isfile(os.path.join(USER_RAYCASTING_PRESETS_DIRECTORY, filename))]
+RAYCASTING_TYPES = [_(filename.name.split(".")[0]) for filename in
+                    inv_paths.USER_RAYCASTING_PRESETS_DIRECTORY.glob('*') if
+                    filename.is_file()]
 RAYCASTING_TYPES += RAYCASTING_FILES.keys()
 RAYCASTING_TYPES.append(_(' Off'))
 RAYCASTING_TYPES.sort()
@@ -440,8 +400,8 @@ RAYCASTING_TOOLS = [_("Cut plane")]
 # If 0 dont't blur, 1 blur
 RAYCASTING_WWWL_BLUR = 0
 
-RAYCASTING_PRESETS_FOLDERS = (RAYCASTING_PRESETS_DIRECTORY,
-                              USER_RAYCASTING_PRESETS_DIRECTORY)
+RAYCASTING_PRESETS_FOLDERS = (inv_paths.RAYCASTING_PRESETS_DIRECTORY,
+                              inv_paths.USER_RAYCASTING_PRESETS_DIRECTORY)
 
 
 ####
@@ -506,10 +466,11 @@ VTK_WARNING = 0
 #----------------------------------------------------------
 
 [ID_DICOM_IMPORT, ID_PROJECT_OPEN, ID_PROJECT_SAVE_AS, ID_PROJECT_SAVE,
-ID_PROJECT_CLOSE, ID_PROJECT_INFO, ID_SAVE_SCREENSHOT, ID_DICOM_LOAD_NET,
-ID_PRINT_SCREENSHOT, ID_IMPORT_OTHERS_FILES, ID_PREFERENCES, ID_DICOM_NETWORK,
-ID_TIFF_JPG_PNG, ID_VIEW_INTERPOLATED, ID_MODE_NAVIGATION, ID_ANALYZE_IMPORT,
-ID_NIFTI_IMPORT, ID_PARREC_IMPORT] = [wx.NewId() for number in range(18)]
+ ID_PROJECT_CLOSE, ID_EXPORT_SLICE, ID_EXPORT_MASK, ID_PROJECT_INFO,
+ ID_SAVE_SCREENSHOT, ID_DICOM_LOAD_NET, ID_PRINT_SCREENSHOT,
+ ID_IMPORT_OTHERS_FILES, ID_PREFERENCES, ID_DICOM_NETWORK, ID_TIFF_JPG_PNG,
+ ID_VIEW_INTERPOLATED, ID_MODE_NAVIGATION, ID_ANALYZE_IMPORT, ID_NIFTI_IMPORT,
+ ID_PARREC_IMPORT, ID_MODE_DBS] = [wx.NewId() for number in range(21)]
 ID_EXIT = wx.ID_EXIT
 ID_ABOUT = wx.ID_ABOUT
 
@@ -546,9 +507,14 @@ ID_WATERSHED_SEGMENTATION = wx.NewId()
 ID_THRESHOLD_SEGMENTATION = wx.NewId()
 ID_FLOODFILL_SEGMENTATION = wx.NewId()
 ID_CROP_MASK = wx.NewId()
+ID_DENSITY_MEASURE = wx.NewId()
+ID_MASK_DENSITY_MEASURE = wx.NewId()
 ID_CREATE_SURFACE = wx.NewId()
 ID_CREATE_MASK = wx.NewId()
 
+ID_GOTO_SLICE = wx.NewId()
+ID_GOTO_COORD = wx.NewId()
+
 #---------------------------------------------------------
 STATE_DEFAULT = 1000
 STATE_WL = 1001
@@ -559,6 +525,9 @@ STATE_PAN = 1005
 STATE_ANNOTATE = 1006
 STATE_MEASURE_DISTANCE = 1007
 STATE_MEASURE_ANGLE = 1008
+STATE_MEASURE_DENSITY = 1009
+STATE_MEASURE_DENSITY_ELLIPSE = 1010
+STATE_MEASURE_DENSITY_POLYGON = 1011
 
 SLICE_STATE_CROSS = 3006
 SLICE_STATE_SCROLL = 3007
@@ -577,7 +546,11 @@ VOLUME_STATE_SEED = 2001
 
 TOOL_STATES = [STATE_WL, STATE_SPIN, STATE_ZOOM,
                STATE_ZOOM_SL, STATE_PAN, STATE_MEASURE_DISTANCE,
-               STATE_MEASURE_ANGLE]  #, STATE_ANNOTATE]
+               STATE_MEASURE_ANGLE, STATE_MEASURE_DENSITY_ELLIPSE,
+               STATE_MEASURE_DENSITY_POLYGON,
+               ]  #, STATE_ANNOTATE]
+
+
 
 TOOL_SLICE_STATES = [SLICE_STATE_CROSS, SLICE_STATE_SCROLL,
                      SLICE_STATE_REORIENT]
@@ -592,6 +565,9 @@ SLICE_STYLES.append(SLICE_STATE_REMOVE_MASK_PARTS)
 SLICE_STYLES.append(SLICE_STATE_SELECT_MASK_PARTS)
 SLICE_STYLES.append(SLICE_STATE_FFILL_SEGMENTATION)
 SLICE_STYLES.append(SLICE_STATE_CROP_MASK)
+SLICE_STYLES.append(STATE_MEASURE_DENSITY)
+SLICE_STYLES.append(STATE_MEASURE_DENSITY_ELLIPSE)
+SLICE_STYLES.append(STATE_MEASURE_DENSITY_POLYGON)
 
 VOLUME_STYLES = TOOL_STATES + [VOLUME_STATE_SEED, STATE_MEASURE_DISTANCE,
         STATE_MEASURE_ANGLE]
@@ -612,6 +588,9 @@ STYLE_LEVEL = {SLICE_STATE_EDITOR: 1,
                STATE_DEFAULT: 0,
                STATE_MEASURE_ANGLE: 2,
                STATE_MEASURE_DISTANCE: 2,
+               STATE_MEASURE_DENSITY_ELLIPSE: 2,
+               STATE_MEASURE_DENSITY_POLYGON: 2,
+               STATE_MEASURE_DENSITY: 2,
                STATE_WL: 2,
                STATE_SPIN: 2,
                STATE_ZOOM: 2,
@@ -684,6 +663,7 @@ STATIC_REF = 0
 DYNAMIC_REF = 1
 DEFAULT_REF_MODE = DYNAMIC_REF
 REF_MODE = [_("Static ref."), _("Dynamic ref.")]
+FT_SENSOR_MODE = [_("Sensor 3"), _("Sensor 4")]
 
 DEFAULT_COIL = SELECT
 COIL = [_("Select coil:"), _("Neurosoft Figure-8"),
@@ -711,13 +691,11 @@ TIPS_IMG = [_("Select left ear in image"),
 
 BTNS_TRK = {TR1: {3: _('LET')},
             TR2: {4: _('RET')},
-            TR3: {5: _('NAT')},
-            SET: {6: _('SET')}}
+            TR3: {5: _('NAT')}}
 
 TIPS_TRK = [_("Select left ear with spatial tracker"),
             _("Select right ear with spatial tracker"),
-            _("Select nasion with spatial tracker"),
-            _("Show set coordinates in image")]
+            _("Select nasion with spatial tracker")]
 
 OBJL = wx.NewId()
 OBJR = wx.NewId()
@@ -737,11 +715,11 @@ TIPS_OBJ = [_("Select left object fiducial"),
             _("Select object center"),
             _("Attach sensor to object")]
 
-CAL_DIR = os.path.abspath(os.path.join(FILE_PATH, '..', 'navigation', 'mtc_files', 'CalibrationFiles'))
-MAR_DIR = os.path.abspath(os.path.join(FILE_PATH, '..', 'navigation', 'mtc_files', 'Markers'))
+PROBE_NAME = "1Probe"
+REF_NAME = "2Ref"
+OBJ_NAME = "3Coil"
 
 #OBJECT TRACKING
-OBJ_DIR = os.path.abspath(os.path.join(FILE_PATH, '..', 'navigation', 'objects'))
 ARROW_SCALE = 3
 ARROW_UPPER_LIMIT = 30
 #COIL_ANGLES_THRESHOLD = 3 * ARROW_SCALE
