@@ -61,6 +61,7 @@ def DefaultTracker(tracker_id):
 
 def ClaronTracker(tracker_id):
     import invesalius.constants as const
+    from invesalius import inv_paths
 
     trck_init = None
     try:
@@ -68,12 +69,13 @@ def ClaronTracker(tracker_id):
 
         lib_mode = 'wrapper'
         trck_init = pyclaron.pyclaron()
-        trck_init.CalibrationDir = const.CAL_DIR.encode(const.FS_ENCODE)
-        trck_init.MarkerDir = const.MAR_DIR.encode(const.FS_ENCODE)
-        trck_init.NumberFramesProcessed = 10
+        trck_init.CalibrationDir = inv_paths.CAL_DIR.encode(const.FS_ENCODE)
+        trck_init.MarkerDir = inv_paths.MAR_DIR.encode(const.FS_ENCODE)
+        trck_init.NumberFramesProcessed = 1
         trck_init.FramesExtrapolated = 0
-        trck_init.PROBE_NAME = "1Probe"
-        trck_init.REF_NAME = "5Ref"
+        trck_init.PROBE_NAME = const.PROBE_NAME.encode(const.FS_ENCODE)
+        trck_init.REF_NAME = const.REF_NAME.encode(const.FS_ENCODE)
+        trck_init.OBJ_NAME = const.OBJ_NAME.encode(const.FS_ENCODE)
         trck_init.Initialize()
 
         if trck_init.GetIdentifyingCamera():
@@ -143,31 +145,26 @@ def PlhWrapperConnection(tracker_id):
 
 
 def PlhSerialConnection(tracker_id):
-    try:
-        import serial
+    import serial
 
-        trck_init = serial.Serial('COM1', baudrate=115200, timeout=0.2)
+    trck_init = serial.Serial('COM3', baudrate=115200, timeout=0.03)
 
-        if tracker_id == 2:
-            # Polhemus FASTRAK needs configurations first
-            trck_init.write(0x02, "u")
-            trck_init.write(0x02, "F")
-        elif tracker_id == 3:
-            # Polhemus ISOTRAK needs to set tracking point from
-            # center to tip.
-            trck_init.write("F")
-            trck_init.write("Y")
+    if tracker_id == 2:
+        # Polhemus FASTRAK needs configurations first
+        trck_init.write(0x02, str.encode("u"))
+        trck_init.write(0x02, str.encode("F"))
+    elif tracker_id == 3:
+        # Polhemus ISOTRAK needs to set tracking point from
+        # center to tip.
+        trck_init.write(str.encode("u"))
+        trck_init.write(str.encode("F"))
+        trck_init.write(str.encode("Y"))
 
-        trck_init.write('P')
-        data = trck_init.readlines()
-
-        if not data:
-            trck_init = None
-            print('Could not connect to Polhemus serial without error.')
-
-    except:
+    trck_init.write(str.encode("P"))
+    data = trck_init.readlines()
+    if not data:
         trck_init = None
-        print('Could not connect to Polhemus serial with error.')
+        print('Could not connect to Polhemus serial without error.')
 
     return trck_init
 
@@ -220,10 +217,16 @@ def DisconnectTracker(tracker_id, trck_init):
         print('Debug tracker disconnected.')
     else:
         try:
-            trck_init.Close()
-            trck_init = False
-            lib_mode = 'wrapper'
-            print('Tracker disconnected.')
+            if tracker_id == 3:
+                trck_init.close()
+                trck_init = False
+                lib_mode = 'serial'
+                print('Tracker disconnected.')
+            else:
+                trck_init.Close()
+                trck_init = False
+                lib_mode = 'wrapper'
+                print('Tracker disconnected.')
         except:
             trck_init = True
             lib_mode = 'error'
