@@ -34,6 +34,9 @@ from invesalius.gui.widgets.clut_raycasting import CLUTRaycastingWidget, \
         EVT_CLUT_CURVE_WL_CHANGE
 
 from invesalius.constants import ID_TO_BMP
+from invesalius import inv_paths
+
+import invesalius.session as ses
 import invesalius.constants as const
 
 class Panel(wx.Panel):
@@ -121,6 +124,9 @@ class Panel(wx.Panel):
 
         self.aui_manager.Update()
 
+        if int(ses.Session().mode) != const.MODE_NAVIGATOR:
+            Publisher.sendMessage('Deactive target button')
+
     def __bind_events_wx(self):
         self.aui_manager.Bind(wx.aui.EVT_AUI_PANE_MAXIMIZE, self.OnMaximize)
         self.aui_manager.Bind(wx.aui.EVT_AUI_PANE_RESTORE, self.OnRestore)
@@ -313,8 +319,6 @@ import wx.lib.buttons as btn
 import wx.lib.pubsub as ps
 import wx.lib.colourselect as csel
 
-import invesalius.constants as const
-
 [BUTTON_RAYCASTING, BUTTON_VIEW, BUTTON_SLICE_PLANE, BUTTON_3D_STEREO, BUTTON_TARGET] = [wx.NewId() for num in range(5)]
 RAYCASTING_TOOLS = wx.NewId()
 
@@ -345,17 +349,17 @@ class VolumeToolPanel(wx.Panel):
         wx.Panel.__init__(self, parent)
 
         # VOLUME RAYCASTING BUTTON
-        BMP_RAYCASTING = wx.Bitmap(os.path.join(const.ICON_DIR, "volume_raycasting.png"),
+        BMP_RAYCASTING = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "volume_raycasting.png"),
                                     wx.BITMAP_TYPE_PNG)
 
-        BMP_SLICE_PLANE = wx.Bitmap(os.path.join(const.ICON_DIR, "slice_plane.png"),
+        BMP_SLICE_PLANE = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "slice_plane.png"),
                                     wx.BITMAP_TYPE_PNG)
 
 
-        BMP_3D_STEREO = wx.Bitmap(os.path.join(const.ICON_DIR, "3D_glasses.png"),
+        BMP_3D_STEREO = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "3D_glasses.png"),
                                     wx.BITMAP_TYPE_PNG)
 
-        BMP_TARGET = wx.Bitmap(os.path.join(const.ICON_DIR, "target.png"),
+        BMP_TARGET = wx.Bitmap(os.path.join(inv_paths.ICON_DIR, "target.png"),
                                     wx.BITMAP_TYPE_PNG)
 
 
@@ -431,6 +435,8 @@ class VolumeToolPanel(wx.Panel):
         Publisher.subscribe(self.DisableVolumeCutMenu, 'Disable volume cut menu')
         Publisher.subscribe(self.StatusTargetSelect, 'Disable or enable coil tracker')
         Publisher.subscribe(self.StatusObjTracker, 'Status target button')
+        Publisher.subscribe(self.ActiveTarget, 'Active target button')
+        Publisher.subscribe(self.DeactiveTarget, 'Deactive target button')
         
     def DisablePreset(self):
         self.off_item.Check(1)
@@ -465,6 +471,12 @@ class VolumeToolPanel(wx.Panel):
         self.status_target_select = status
         self.StatusNavigation()
 
+    def ActiveTarget(self):
+        self.button_target.Show()
+
+    def DeactiveTarget(self):
+        self.button_target.Hide()
+
     def StatusNavigation(self):
         if self.status_target_select and self.status_obj_tracker:
             self.button_target.Enable(1)
@@ -495,7 +507,7 @@ class VolumeToolPanel(wx.Panel):
         for name in const.RAYCASTING_TYPES:
             id = wx.NewId()
             item = wx.MenuItem(menu, id, name, kind=wx.ITEM_RADIO)
-            menu.AppendItem(item)
+            menu.Append(item)
             if name == const.RAYCASTING_OFF_LABEL:
                 self.off_item = item
                 item.Check(1)
@@ -511,12 +523,12 @@ class VolumeToolPanel(wx.Panel):
                self.id_cutplane = id
            
            item = wx.MenuItem(submenu, id, name, kind=wx.ITEM_CHECK)
-           submenu.AppendItem(item)
+           submenu.Append(item)
            ID_TO_TOOL[id] = name
            ID_TO_TOOL_ITEM[id] = item
            TOOL_STATE[id] = False
         self.submenu_raycasting_tools = submenu
-        menu.AppendMenu(RAYCASTING_TOOLS, _("Tools"), submenu)
+        menu.Append(RAYCASTING_TOOLS, _("Tools"), submenu)
         menu.Enable(RAYCASTING_TOOLS, 0)
 
         self.menu_raycasting = menu
@@ -533,7 +545,7 @@ class VolumeToolPanel(wx.Panel):
             bmp =  wx.Bitmap(ID_TO_BMP[id][1], wx.BITMAP_TYPE_PNG)
             item = wx.MenuItem(menu, id, ID_TO_BMP[id][0])
             item.SetBitmap(bmp)
-            menu.AppendItem(item)
+            menu.Append(item)
         menu.Bind(wx.EVT_MENU, self.OnMenuView)
         self.menu_view = menu
 
@@ -547,7 +559,7 @@ class VolumeToolPanel(wx.Panel):
             item = wx.MenuItem(slice_plane_menu, new_id, value,
                                             kind = wx.ITEM_CHECK)
             ID_TO_ITEMSLICEMENU[new_id] = item
-            slice_plane_menu.AppendItem(item)
+            slice_plane_menu.Append(item)
 
         slice_plane_menu.Bind(wx.EVT_MENU, self.OnMenuPlaneSlice)
 
@@ -566,7 +578,7 @@ class VolumeToolPanel(wx.Panel):
 
             ID_TO_ITEM_3DSTEREO[new_id] = item
             ID_TO_STEREO_NAME[new_id] = value 
-            stereo_menu.AppendItem(item)
+            stereo_menu.Append(item)
 
         stereo_menu.Bind(wx.EVT_MENU, self.OnMenuStereo)
 
@@ -622,7 +634,7 @@ class VolumeToolPanel(wx.Panel):
                                   preset_name=ID_TO_NAME[id])
             # Enable or disable tools
             if name != const.RAYCASTING_OFF_LABEL:
- 	            self.menu_raycasting.Enable(RAYCASTING_TOOLS, 1)
+                self.menu_raycasting.Enable(RAYCASTING_TOOLS, 1)
             else:
                 self.menu_raycasting.Enable(RAYCASTING_TOOLS, 0)
 
