@@ -134,13 +134,9 @@ class InnerFoldPanel(wx.Panel):
         # is not working properly in this panel. It might be on some child or
         # parent panel. Perhaps we need to insert the item into the sizer also...
         # Study this.
-        displaySize = wx.DisplaySize()
-        if displaySize[1] > 768:
-            fold_panel = fpb.FoldPanelBar(self, -1, wx.DefaultPosition,
-                                          (10, 350), 0, fpb.FPB_SINGLE_FOLD)
-        else:
-            fold_panel = fpb.FoldPanelBar(self, -1, wx.DefaultPosition,
-                                          (10, 320), 0, fpb.FPB_SINGLE_FOLD)
+
+        fold_panel = fpb.FoldPanelBar(self, -1, wx.DefaultPosition,
+                                          (10, 290), 0, fpb.FPB_SINGLE_FOLD)
         # Fold panel style
         style = fpb.CaptionBarStyle()
         style.SetCaptionStyle(fpb.CAPTIONBAR_GRADIENT_V)
@@ -172,6 +168,15 @@ class InnerFoldPanel(wx.Panel):
         fold_panel.AddFoldPanelWindow(item, mtw, spacing= 0,
                                       leftSpacing=0, rightSpacing=0)
 
+        # Fold 4 - DBS
+
+        self.dbs_item = fold_panel.AddFoldPanel(_("Deep Brain Stimulation"), collapsed=True)
+        dtw = DbsPanel(self.dbs_item) #Atribuir nova var, criar panel
+
+        fold_panel.ApplyCaptionStyle(self.dbs_item, style)
+        fold_panel.AddFoldPanelWindow(self.dbs_item, dtw, spacing= 0,
+                                      leftSpacing=0, rightSpacing=0)
+        self.dbs_item.Hide()
 
         # Check box for camera update in volume rendering during navigation
         tooltip = wx.ToolTip(_("Update camera in volume"))
@@ -220,11 +225,21 @@ class InnerFoldPanel(wx.Panel):
         self.SetSizer(sizer)
         self.Update()
         self.SetAutoLayout(1)
-
+        
     def __bind_events(self):
         Publisher.subscribe(self.OnCheckStatus, 'Navigation status')
         Publisher.subscribe(self.OnShowObject, 'Update track object state')
         Publisher.subscribe(self.OnVolumeCamera, 'Target navigation mode')
+        Publisher.subscribe(self.OnShowDbs, "Active dbs folder")
+        Publisher.subscribe(self.OnHideDbs, "Deactive dbs folder")
+
+    def OnShowDbs(self):
+        self.dbs_item.Show()
+
+
+    def OnHideDbs(self):
+        self.dbs_item.Hide()
+
 
     def OnCheckStatus(self, status):
         if status:
@@ -258,7 +273,6 @@ class InnerFoldPanel(wx.Panel):
                 self.checkcamera.SetValue(0)
         Publisher.sendMessage('Update volume camera state', camera_state=self.checkcamera.GetValue())
 
-
 class NeuronavigationPanel(wx.Panel):
     def __init__(self, parent):
         wx.Panel.__init__(self, parent)
@@ -287,8 +301,8 @@ class NeuronavigationPanel(wx.Panel):
         self.ref_mode_id = const.DEFAULT_REF_MODE
 
         # Initialize list of buttons and numctrls for wx objects
-        self.btns_coord = [None] * 7
-        self.numctrls_coord = [list(), list(), list(), list(), list(), list(), list()]
+        self.btns_coord = [None, None, None, None, None, None]
+        self.numctrls_coord = [[], [], [], [], [], []]
 
         # ComboBox for spatial tracker device selection
         tooltip = wx.ToolTip(_("Choose the tracking device"))
@@ -327,11 +341,7 @@ class NeuronavigationPanel(wx.Panel):
             lab = list(btns_trk[k].values())[0]
             self.btns_coord[n] = wx.Button(self, k, label=lab, size=wx.Size(45, 23))
             self.btns_coord[n].SetToolTip(wx.ToolTip(tips_trk[n-3]))
-            # Exception for event of button that set image coordinates
-            if n == 6:
-                self.btns_coord[n].Bind(wx.EVT_BUTTON, self.OnSetImageCoordinates)
-            else:
-                self.btns_coord[n].Bind(wx.EVT_BUTTON, self.OnTrackerFiducials)
+            self.btns_coord[n].Bind(wx.EVT_BUTTON, self.OnTrackerFiducials)
 
         # TODO: Find a better allignment between FRE, text and navigate button
         txt_fre = wx.StaticText(self, -1, _('FRE:'))
@@ -352,8 +362,8 @@ class NeuronavigationPanel(wx.Panel):
         btn_nav.Bind(wx.EVT_TOGGLEBUTTON, partial(self.OnNavigate, btn=(btn_nav, choice_trck, choice_ref)))
 
         # Image and tracker coordinates number controls
-        for m in range(0, 7):
-            for n in range(0, 3):
+        for m in range(len(self.btns_coord)):
+            for n in range(3):
                 self.numctrls_coord[m].append(
                     wx.lib.masked.numctrl.NumCtrl(parent=self, integerWidth=4, fractionWidth=1))
 
@@ -364,9 +374,9 @@ class NeuronavigationPanel(wx.Panel):
 
         coord_sizer = wx.GridBagSizer(hgap=5, vgap=5)
 
-        for m in range(0, 7):
+        for m in range(len(self.btns_coord)):
             coord_sizer.Add(self.btns_coord[m], pos=wx.GBPosition(m, 0))
-            for n in range(0, 3):
+            for n in range(3):
                 coord_sizer.Add(self.numctrls_coord[m][n], pos=wx.GBPosition(m, n+1))
                 if m in range(1, 6):
                     self.numctrls_coord[m][n].SetEditable(False)
@@ -414,13 +424,8 @@ class NeuronavigationPanel(wx.Panel):
     def UpdateImageCoordinates(self, position):
         # TODO: Change from world coordinates to matrix coordinates. They are better for multi software communication.
         self.current_coord = position
-        for m in [0, 1, 2, 6]:
-            if m == 6 and self.btns_coord[m].IsEnabled():
-                for n in [0, 1, 2]:
-                    self.numctrls_coord[m][n].SetValue(float(self.current_coord[n]))
-            elif m != 6 and not self.btns_coord[m].GetValue():
-                # btn_state = self.btns_coord[m].GetValue()
-                # if not btn_state:
+        for m in [0, 1, 2]:
+            if not self.btns_coord[m].GetValue():
                 for n in [0, 1, 2]:
                     self.numctrls_coord[m][n].SetValue(float(self.current_coord[n]))
 
@@ -473,12 +478,11 @@ class NeuronavigationPanel(wx.Panel):
                     ctrl.SetSelection(0)
                     print("Tracker not connected!")
                 else:
-                    Publisher.sendMessage('Update status text in GUI',
-                                          label=_("Ready"))
                     ctrl.SetSelection(self.tracker_id)
                     print("Tracker connected!")
         elif choice == 6:
             if trck:
+                self.ResetTrackerFiducials()
                 Publisher.sendMessage('Update status text in GUI',
                                       label=_("Disconnecting tracker ..."))
                 Publisher.sendMessage('Remove sensors ID')
@@ -507,10 +511,8 @@ class NeuronavigationPanel(wx.Panel):
                     dlg.NavigationTrackerWarning(self.tracker_id, self.trk_init[1])
                     self.tracker_id = 0
                     ctrl.SetSelection(self.tracker_id)
-                else:
-                    Publisher.sendMessage('Update status text in GUI',
-                                          label=_("Ready"))
 
+        Publisher.sendMessage('Update status text in GUI', label=_("Ready"))
         Publisher.sendMessage('Update tracker initializer',
                               nav_prop=(self.tracker_id, self.trk_init, self.ref_mode_id))
 
@@ -525,19 +527,6 @@ class NeuronavigationPanel(wx.Panel):
                               nav_prop=(self.tracker_id, self.trk_init, self.ref_mode_id))
         print("Reference mode changed!")
 
-    def OnSetImageCoordinates(self, evt):
-        # FIXME: Cross does not update in last clicked slice, only on the other two
-        btn_id = list(const.BTNS_TRK[evt.GetId()].keys())[0]
-
-        ux, uy, uz = self.numctrls_coord[btn_id][0].GetValue(),\
-                     self.numctrls_coord[btn_id][1].GetValue(),\
-                     self.numctrls_coord[btn_id][2].GetValue()
-
-        Publisher.sendMessage('Set ball reference position', position=(ux, uy, uz))
-        # Publisher.sendMessage('Set camera in volume', (ux, uy, uz))
-        Publisher.sendMessage('Co-registered points', arg=(ux, uy, uz), position=(0., 0., 0.))
-        Publisher.sendMessage('Update cross position', position=(ux, uy, uz))
-
     def OnImageFiducials(self, evt):
         btn_id = list(const.BTNS_IMG_MKS[evt.GetId()].keys())[0]
         marker_id = list(const.BTNS_IMG_MKS[evt.GetId()].values())[0]
@@ -695,9 +684,6 @@ class NeuronavigationPanel(wx.Panel):
             for n in range(0, 3):
                 self.numctrls_coord[m][n].SetValue(0.0)
 
-        for n in range(0, 3):
-            self.numctrls_coord[6][n].SetValue(0.0)
-
     def ResetTrackerFiducials(self):
         for m in range(3, 6):
             self.fiducials[m, :] = [np.nan, np.nan, np.nan]
@@ -1078,8 +1064,12 @@ class MarkersPanel(wx.Panel):
         menu_id = wx.Menu()
         edit_id = menu_id.Append(0, _('Edit ID'))
         menu_id.Bind(wx.EVT_MENU, self.OnMenuEditMarkerId, edit_id)
+        color_id = menu_id.Append(2, _('Edit color'))
+        menu_id.Bind(wx.EVT_MENU, self.OnMenuSetColor, color_id)
+        menu_id.AppendSeparator()
         target_menu = menu_id.Append(1, _('Set as target'))
         menu_id.Bind(wx.EVT_MENU, self.OnMenuSetTarget, target_menu)
+
         target_menu.Enable(status)
         self.PopupMenu(menu_id)
         menu_id.Destroy()
@@ -1099,7 +1089,7 @@ class MarkersPanel(wx.Panel):
             if id_label == 'TARGET':
                 id_label = ''
                 dlg.InvalidTargetID()
-        self.lc.SetStringItem(list_index, 4, id_label)
+        self.lc.SetItem(list_index, 4, id_label)
         # Add the new ID to exported list
         if len(self.list_coord[list_index]) > 8:
             self.list_coord[list_index][10] = str(id_label)
@@ -1113,7 +1103,7 @@ class MarkersPanel(wx.Panel):
         if self.tgt_flag:
             self.lc.SetItemBackgroundColour(self.tgt_index, 'white')
             Publisher.sendMessage('Set target transparency', status=False, index=self.tgt_index)
-            self.lc.SetStringItem(self.tgt_index, 4, '')
+            self.lc.SetItem(self.tgt_index, 4, '')
             # Add the new ID to exported list
             if len(self.list_coord[self.tgt_index]) > 8:
                 self.list_coord[self.tgt_index][10] = str('')
@@ -1130,6 +1120,26 @@ class MarkersPanel(wx.Panel):
         self.tgt_flag = True
         dlg.NewTarget()
 
+    def OnMenuSetColor(self, evt):
+        index = self.lc.GetFocusedItem()
+        cdata = wx.ColourData()
+        cdata.SetColour(wx.Colour(self.list_coord[index][6]*255,self.list_coord[index][7]*255,self.list_coord[index][8]*255))
+        dlg = wx.ColourDialog(self, data=cdata)
+        dlg.GetColourData().SetChooseFull(True)
+        if dlg.ShowModal() == wx.ID_OK:
+            self.r, self.g, self.b = dlg.GetColourData().GetColour().Get(includeAlpha=False)
+            r = float(self.r) / 255.0
+            g = float(self.g) / 255.0
+            b = float(self.b) / 255.0
+        dlg.Destroy()
+        color = [r,g,b]
+
+        Publisher.sendMessage('Set new color', index=index, color=color)
+
+        self.list_coord[index][6] = r
+        self.list_coord[index][7] = g
+        self.list_coord[index][8] = b
+
     def OnDeleteAllMarkers(self, evt=None):
         if self.list_coord:
             if evt is None:
@@ -1183,7 +1193,7 @@ class MarkersPanel(wx.Panel):
             del self.list_coord[i]
             self.lc.DeleteItem(i)
             for n in range(0, self.lc.GetItemCount()):
-                self.lc.SetStringItem(n, 0, str(n+1))
+                self.lc.SetItem(n, 0, str(n+1))
             self.marker_ind -= 1
         Publisher.sendMessage('Remove marker', index=index)
 
@@ -1298,11 +1308,11 @@ class MarkersPanel(wx.Panel):
 
         # Add item to list control in panel
         num_items = self.lc.GetItemCount()
-        self.lc.InsertStringItem(num_items, str(num_items + 1))
-        self.lc.SetStringItem(num_items, 1, str(round(coord[0], 2)))
-        self.lc.SetStringItem(num_items, 2, str(round(coord[1], 2)))
-        self.lc.SetStringItem(num_items, 3, str(round(coord[2], 2)))
-        self.lc.SetStringItem(num_items, 4, str(marker_id))
+        self.lc.InsertItem(num_items, str(num_items + 1))
+        self.lc.SetItem(num_items, 1, str(round(coord[0], 2)))
+        self.lc.SetItem(num_items, 2, str(round(coord[1], 2)))
+        self.lc.SetItem(num_items, 3, str(round(coord[2], 2)))
+        self.lc.SetItem(num_items, 4, str(marker_id))
         self.lc.EnsureVisible(num_items)
 
     def GetSelectedItems(self):
@@ -1316,3 +1326,12 @@ class MarkersPanel(wx.Panel):
             index = self.lc.GetNextSelected(index)
             selection.append(index)
         return selection
+
+class DbsPanel(wx.Panel):
+    def __init__(self, parent):
+        wx.Panel.__init__(self, parent)
+        try:
+            default_colour = wx.SystemSettings.GetColour(wx.SYS_COLOUR_MENUBAR)
+        except AttributeError:
+            default_colour = wx.SystemSettings_GetColour(wx.SYS_COLOUR_MENUBAR)
+
